"""
Pylint plugin
"""

from astroid import MANAGER, ClassDef

VERSION = "0.1.0"


def register(linter):  # pylint: disable=unused-argument
    """Plugin registration"""


def transform(cls):
    """
    Mimics Flask-SQLAlchemy's _include_sqlalchemy
    """
    if cls.name == "SQLAlchemy":
        import sqlalchemy
        import sqlalchemy.orm

        for module in sqlalchemy, sqlalchemy.orm:
            for key in module.__all__:
                cls.locals[key] = [ClassDef(key, None)]
    if cls.name == "scoped_session":
        from sqlalchemy.orm import Session

        for key in Session.public_methods:
            cls.locals[key] = [ClassDef(key, None)]


MANAGER.register_transform(ClassDef, transform)



"""Tests"""
import pytest
from pylint.lint import Run

# pylint: disable=missing-docstring

E1 = "E1101: Instance of 'SQLAlchemy' has no 'Column' member (no-member)"
E2 = "E1101: Instance of 'SQLAlchemy' has no 'Integer' member (no-member)"
E3 = "E1101: Instance of 'SQLAlchemy' has no 'String' member (no-member)"
E4 = "E1101: Instance of 'scoped_session' has no 'add' member (no-member)"
E5 = "E1101: Instance of 'scoped_session' has no 'commit' member (no-member)"
PARAMS = ["tests/example.py", "--errors-only", "--exit-zero"]


@pytest.mark.parametrize("error", [E1, E2, E3, E4, E5])
def test_without_plugin(capsys, error):
    try:
        Run(PARAMS)
    except SystemExit:
        pass
    captured = capsys.readouterr()
    for message in captured.out.split("\n"):
        assert error in message


@pytest.mark.parametrize("error", [E1, E2, E3, E4, E5])
def test_with_plugin(capsys, error):
    try:
        Run(
            [
                "tests/example.py",
                "--errors-only",
                "--exit-zero",
                "--load-plugins",
                "pylint_flask_sqlalchemy",
            ]
        )
    except SystemExit:
        pass
    captured = capsys.readouterr()
    for message in captured.out.split("\n"):
        assert error not in message



# pylint: disable=missing-docstring,too-few-public-methods,invalid-name
from flask import Flask
from flask_sqlalchemy import SQLAlchemy

app = Flask(__name__)
app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:////tmp/test.db"
db = SQLAlchemy(app)


class User(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(80), unique=True, nullable=False)

    def __repr__(self):
        return "<User %r>" % self.username


user = User(username="test")
db.session.add(user)
db.session.commit()
